@@ -16,9 +16,10 @@ use crate::commands::helpers::progress_spinner;
 use crate::crypto::hash;
 use crate::id::Id;
 use crate::index::{IndexBackend, IndexCollector, IndexType, IndexedBackend};
-use crate::repo::{
+use crate::repofile::{
     IndexFile, IndexPack, PackHeader, PackHeaderLength, PackHeaderRef, SnapshotFile,
 };
+use crate::repository::OpenRepository;
 
 #[derive(Parser)]
 pub(super) struct Opts {
@@ -31,13 +32,11 @@ pub(super) struct Opts {
     read_data: bool,
 }
 
-pub(super) fn execute(
-    be: &impl DecryptReadBackend,
-    cache: &Option<Cache>,
-    hot_be: &Option<impl ReadBackend>,
-    raw_be: &impl ReadBackend,
-    opts: Opts,
-) -> Result<()> {
+pub(super) fn execute(repo: OpenRepository, opts: Opts) -> Result<()> {
+    let be = &repo.dbe;
+    let cache = &repo.cache;
+    let hot_be = &repo.be_hot;
+    let raw_be = &repo.be;
     if !opts.trust_cache {
         if let Some(cache) = &cache {
             for file_type in [FileType::Snapshot, FileType::Index] {
@@ -140,7 +139,7 @@ fn check_cache_files(
         return Ok(());
     }
 
-    let total_size = files.iter().map(|(_, size)| *size as u64).sum();
+    let total_size = files.values().map(|size| *size as u64).sum();
     p.set_length(total_size);
 
     files.into_par_iter()
