@@ -2,8 +2,9 @@ use anyhow::{bail, Result};
 use clap::Parser;
 use indicatif::ProgressBar;
 
-use crate::backend::{DecryptReadBackend, FileType};
-use crate::repo::IndexFile;
+use crate::backend::{DecryptReadBackend, FileType, ReadBackend};
+use crate::repofile::IndexFile;
+use crate::repository::OpenRepository;
 
 #[derive(Parser)]
 pub(super) struct Opts {
@@ -12,11 +13,11 @@ pub(super) struct Opts {
     tpe: String,
 }
 
-pub(super) fn execute(be: &impl DecryptReadBackend, opts: Opts) -> Result<()> {
+pub(super) fn execute(repo: OpenRepository, opts: Opts) -> Result<()> {
     let tpe = match opts.tpe.as_str() {
         // special treatment for listing blobs: read the index and display it
         "blobs" => {
-            for index in be.stream_all::<IndexFile>(ProgressBar::hidden())? {
+            for index in repo.dbe.stream_all::<IndexFile>(ProgressBar::hidden())? {
                 for pack in index?.1.packs {
                     for blob in pack.blobs {
                         println!("{:?} {}", blob.tpe, blob.id.to_hex());
@@ -32,7 +33,7 @@ pub(super) fn execute(be: &impl DecryptReadBackend, opts: Opts) -> Result<()> {
         t => bail!("invalid type: {}", t),
     };
 
-    for id in be.list(tpe)? {
+    for id in repo.be.list(tpe)? {
         println!("{}", id.to_hex());
     }
 
